#Project template for CHE210D
#Brandon Knott
#due: 6/10/2009

#import python modules
import numpy as np
import time
import atomwrite

#import compiled fortran library
import movieproject
import sys
import os

#========== GLOBAL VARIABLES ==========
N=125
NMax=125
debye=20.   #in angstroms
L = 3.*debye/3.487  #box side length
V=L**3
Cut = 0.5*L    #cutoff distance for LJ interactions
MaxMoveNa=1.33/3.487
MaxMoveCl=2.03/3.487
np.random.seed(342324)

def InitNucleus(N,L):
    Pos = np.zeros((NMax,3), float)
    AtomTypes = np.zeros((NMax,1),int)
    NLat = 5
    LatSpac = 1.54 #make an array of lattice sites (1.54 gives lowest initial energy)
    r = LatSpac * np.arange(NLat, dtype=float)
    #loop through x, y, z positions in lattice until done
    #for every atom in the system
    i = 0
    for x in r:
        for y in r:
            for z in r:
                Pos[i] = np.array([x,y,z], float)
                AtomTypes[i] = -(-1)**i
                i += 1
                #if done placing atoms, return
                if i >= N:
                    return Pos,AtomTypes
    return Pos,AtomTypes    

def Displace(Pos,AtomTypes,PEnergy):    #move an atom
    MoveAtom=np.random.randint(N)
    if AtomTypes[MoveAtom] < 0:
        MaxMove=MaxMoveCl
    else:
        MaxMove=MaxMoveNa
    OldPos=Pos[MoveAtom].copy()
    SingleEOld=movieproject.singleenergy(Pos,AtomTypes,MoveAtom,N,L,Cut)
    Pos[MoveAtom]=Pos[MoveAtom]+MaxMove*(2*np.random.rand(3)-1.0)
    Pos[MoveAtom] -= L*np.around(Pos[MoveAtom]/L)
    SingleENew=movieproject.singleenergy(Pos,AtomTypes,MoveAtom,N,L,Cut)
    if SingleENew<SingleEOld or np.exp(-(SingleENew-SingleEOld))>np.random.random():   #accept the displacement
        PEnergy=PEnergy-SingleEOld+SingleENew
    else:       #move the particle back
        Pos[MoveAtom]=OldPos

    return Pos,PEnergy

#### MODULE LEVEL CODE ####
TotalSweeps=180
UpdateFreq=20
z=file("outputA.txt","w")
z.write('sweep \t PEnergy \t N \t NetCharge \t BigCluster \n')
BetaMu=-2.2
Pos,AtomTypes=InitNucleus(N,L)
WriteFreq=1
Names=[]
for c in range(N):
    if (AtomTypes[c])==-1:
        Names.append("Cl")
    else:
        Names.append("Na")
Pdb = atomwrite.pdbfile("anim.pdb", L, AtomNames=Names)
PEnergy=movieproject.calcenergy(Pos,AtomTypes,N,L,Cut)
for sweep in range(TotalSweeps):
    if sweep%UpdateFreq==0:    #update total energy
        PEnergy=movieproject.calcenergy(Pos,AtomTypes,N,L,Cut)
    if sweep%WriteFreq==0:
        Pdb.write(Pos)
        Charge=np.sum(AtomTypes)
        z.write(str(sweep))
        z.write('\t')
        z.write(str(PEnergy))
        z.write('\t')
        z.write(str(N))
        z.write('\t')
        z.write(str(Charge))
        z.write('\t')
        z.write('\n')
        print sweep,  N, PEnergy
    for move in range(125):
        Pos,PEnergy = Displace(Pos,AtomTypes,PEnergy)
z.close()
Pdb.close()

