
module constants

    real (kind=8),parameter:: T=298., pi=3.141592653, lumped=0.000209985 !lumped = q^2/(eps0*kb)
    real (kind=8),parameter:: epss=78., epsClCl= 117.7604, sigmaClCl=3.487, rclust=2.0
    real (kind=8),parameter:: debye=20./sigmaClCl, rme=2.70/sigmaClCl,rmh=3.1/sigmaClCl
    real (kind=8),parameter:: H=4100./(T*sigmaClCl),sigmae=0.57/sigmaClCl,sigmah=sigmae
    real (kind=8),parameter:: Hydropre=H/(sigmah*SQRT(2.*pi)),Yukpre=lumped/(4.*pi*T)
    integer, parameter:: NAtomMax=125

end module constants

subroutine SingleEnergy(Pos, AtomTypes, MoveAtom, NAtom, L, rc, SingleE, Dim)
    use constants
    implicit none
    integer, intent(in)::Dim, NAtom, MoveAtom
    real(8), intent(in), dimension(0:NAtomMax-1, 0:Dim-1)::Pos     
    integer, intent(in), dimension(0:NAtomMax-1)::AtomTypes
    real(8), intent(in) :: L, rc
    real(8), intent(out) ::SingleE
    real(8), dimension(Dim):: PosMoveAtom,disvec
    real(8):: rc2,rij2,rij,epsd,LJpre1,LJpre2
    real(8):: invrij2, invrij6, invrij12,LJ,Yukawa,Hydro,epskb,sigma
    integer:: i, interact
    !subroutine computes the interaction energy of a single particle with all others
    rc2=rc*rc
    SingleE = 0.
    PosMoveAtom=Pos(MoveAtom,:)
    do i=0,NAtom-1
        if(AtomTypes(MoveAtom)>0 .and. AtomTypes(i)>0) then !Na-Na
            epskb=14.4388/T
            sigma=2.443/sigmaClCl
            interact=1
        elseif(AtomTypes(MoveAtom)<0 .and. AtomTypes(i)<0)then !Cl-Cl
            epskb=117.7604/T
            sigma=3.487/sigmaClCl
            interact=1
        else    !Na-Cl
            epskb=42.4104/T
            sigma=2.796/sigmaClCl
            interact=-1
        endif
        LJpre1=4.*epskb*sigma**12
        LJpre2=4.*epskb*sigma**6
        disvec=Pos(i,:)-PosMoveAtom
        disvec=disvec-L*dnint(disvec/L) !periodic BCs
        rij2=sum(disvec*disvec)
        if(MoveAtom.ne.i)then
            rij=dsqrt(rij2)
            epsd=(5.2+epss)*0.5+(epss-5.2)*0.5*tanh((rij-rme)/sigmae)
            Yukawa=interact*Yukpre*dexp(-rij/debye)/(rij*epsd)
            Hydro=Hydropre*dexp(-(rij-rmh)**2/(2*sigmah**2))
            if(rij2.LT.rc2)then
                invrij2=1./rij2                 !inverse distance squared
                invrij6=invrij2*invrij2*invrij2 !inverse sixth distance
                invrij12=invrij6*invrij6        !inverse twelfth distance
                LJ=LJpre1*invrij12-LJpre2*invrij6   
            endif    
            
            SingleE=SingleE+LJ+Yukawa+Hydro            
        endif
    enddo

end subroutine

subroutine CalcEnergy(Pos, AtomTypes, NAtom, L, rc, PEnergy, Dim)
    use constants
    implicit none
    integer, intent(in) :: Dim, NAtom
    real(8), intent(in), dimension(0:NAtomMax-1, 0:Dim-1) :: Pos
    integer, intent(in), dimension(0:NAtomMax-1)::AtomTypes
    real(8), intent(in) :: L, rc
    real(8), intent(out) :: PEnergy
    real(8), dimension(Dim)::Posi,disvec
    real(8):: rc2,rij2,rij,epskb,sigma,LJpre1,LJpre2
    real(8):: invrij2,invrij6,invrij12,LJ,Yukawa,Hydro,epsd
    integer:: i,j, interact
    !======== BK's CODE HERE ========
    !copy and edit the code for CalcEnergyForces  to compute only the total potential energy.
    !need to pass in the temperature
    rc2=rc*rc
    PEnergy = 0.
    do i=0, NAtom-1
        Posi=Pos(i,:)
        do j=i+1, NAtom-1
            if(AtomTypes(j)>0 .and. AtomTypes(i)>0) then !Na-Na
                epskb=14.4388/T
                sigma=2.443/sigmaClCl
                interact=1
            elseif(AtomTypes(j)<0 .and. AtomTypes(i)<0)then !Cl-Cl
                epskb=117.7604/T
                sigma=3.487/sigmaClCl
                interact=1
            else    !Na-Cl
                epskb=42.4104/T
                sigma=2.796/sigmaClCl
                interact=-1
            endif
            LJpre1=4.*epskb*sigma**12
            LJpre2=4.*epskb*sigma**6
            disvec=Pos(j,:)-Posi     !a vector with the difference in each of the cartesian coords
            disvec=disvec-L*dnint(disvec/L) !periodic BC's
            rij2=Sum(disvec*disvec)   !the squared distance between i and j
            rij=dsqrt(rij2)
            epsd=(5.2+epss)*0.5+(epss-5.2)*0.5*tanh((rij-rme)/sigmae)  
            Yukawa=interact*Yukpre*dexp(-rij/debye)/(rij*epsd)
            Hydro=Hydropre*dexp(-(rij-rmh)**2/(2*sigmah**2))         
            if(rij2.LT.rc2)then
                invrij2=1./rij2                 !inverse distance squared
                invrij6=invrij2*invrij2*invrij2 !inverse sixth distance
                invrij12=invrij6*invrij6        !inverse twelfth distance
                LJ=LJpre1*invrij12-LJpre2*invrij6   
             endif
            
            PEnergy=PEnergy+LJ+Yukawa+Hydro
        enddo
    enddo 
    write(*,*) PEnergy, LJ, Yukawa, Hydro
end subroutine

subroutine Cluster(Pos,NAtom,L,MaxClust,Dim)
    use constants
    implicit none
    integer,intent(in):: Dim
    integer,intent(in):: NAtom
    real(8), intent(in), dimension(0:NAtomMax-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L
    integer,intent(out):: MaxClust
    real(8), dimension(Dim)::Posi,disvec    
    real(8):: rclust2,rij2
    integer:: i,j,p,q,z,clustersize(0:NAtomMax-1),clusternum(0:NAtomMax-1)
    integer:: nextlist(0:NAtomMax-1),list(0:NAtomMax-1),clustercounter
    logical:: loop
    !initialize
    clustercounter=0    !size of the current cluster
    clustersize=0   !array of clustersizes
    clusternum=0    !array of which cluster each atom belongs to
    list=0
    nextlist=0
    loop=.true.
    i=-1
    p=0
    q=0
    rclust2=rclust*rclust
    MaxClust=0
    if(NAtom.gt.0)then
        do while(loop)
            if(sum(list).ne.0)then
                i=list(q)
                list(q)=0
                q=q+1
            else
                i=i+1
                if(i.ge.NAtom)then
                    i=0
                endif
            endif
            if(clusternum(i).eq.0) then
                clustercounter=clustercounter+1
                clusternum(i)=clustercounter
                clustersize(clustercounter)=1      
            endif   
            Posi=Pos(i,:)
            do j=0,NAtom-1
                !if(j.ne.i)then
                if(clusternum(j).eq.0) then
                    disvec=Pos(j,:)-Posi     !a vector with the difference in each of the cartesian coords
                    disvec=disvec-L*dnint(disvec/L) !periodic BC's
                    rij2=Sum(disvec*disvec)   !the squared distance between i and j                    
                    if(rij2.le.rclust2)then
                        clusternum(j)=clusternum(i)
                        clustersize(clustercounter)=clustersize(clustercounter)+1
                        nextlist(p)=j
                        p=p+1
                    endif
                endif
                !endif
            enddo   
            MaxClust=max(MaxClust,clustersize(clustercounter))
            if(sum(list).eq.0)then
                list=nextlist
                nextlist=0
                p=0
                q=0
            endif
            loop=.false.
            do z=0,NAtom-1     !the subroutine stops when every atom is assigned a cluster
                if(clusternum(z).eq.0)then   
                    loop=.true.
                endif    
            enddo
        enddo
    endif
end subroutine