#Project template for CHE210D
#Brandon Knott
#due: 6/10/2009

#import python modules
import numpy as np
import time
import atomwrite

#import compiled fortran library
import project
import sys
import os

#========== GLOBAL VARIABLES ==========
NMax=10000
debye=20.   #in angstroms
L = 3.*debye/3.487  #box side length
V=L**3
Cut = 0.5*L    #cutoff distance for LJ interactions
MaxMoveNa=1.33/3.487
MaxMoveCl=2.03/3.487
np.random.seed(342324)

def InitRandom(N,L):
    Pos = np.zeros((NMax,3), float)
    AtomTypes = np.zeros((NMax,1),int)
    #compute integer grid # of locations for cubic lattice
    NLat = int(N**(1./3.) + 1.)
    LatSpac = L / NLat
    #make an array of lattice sites
    r = LatSpac * np.arange(NLat, dtype=float) - 0.5*L
    #loop through x, y, z positions in lattice until done
    #for every atom in the system
    i = 0
    for x in r:
        for y in r:
            for z in r:
                Pos[i] = np.array([x,y,z], float)
                AtomTypes[i] = -(-1)**i
                #add a random offset to help initial minimization
                Offset = 0.3 * LatSpac * (np.random.rand(3) - 0.5)
                Pos[i] = Pos[i] + Offset
                i += 1
                #if done placing atoms, return
                if i >= N:
                    return Pos,AtomTypes
    return Pos,AtomTypes
    
def InitNucleus(N,L):
    Pos = np.zeros((NMax,3), float)
    AtomTypes = np.zeros((NMax,1),int)
    NLat = int(N**(1./3.) + 1.)
    LatSpac = 1.54 #make an array of lattice sites (1.54 gives lowest initial energy)
    r = LatSpac * np.arange(NLat, dtype=float)
    #loop through x, y, z positions in lattice until done
    #for every atom in the system
    i = 0
    for x in r:
        for y in r:
            for z in r:
                Pos[i] = np.array([x,y,z], float)
                AtomTypes[i] = -(-1)**i
                i += 1
                #if done placing atoms, return
                if i >= N:
                    return Pos,AtomTypes
    return Pos,AtomTypes    

def Displace(Pos,AtomTypes,PEnergy,windowmin,windowmax,BigClust):    #move an atom
    OldBigClust=BigClust
    MoveAtom=np.random.randint(N)
    if AtomTypes[MoveAtom] < 0:
        MaxMove=MaxMoveCl
    else:
        MaxMove=MaxMoveNa
    OldPos=Pos[MoveAtom].copy()
    SingleEOld=project.singleenergy(Pos,AtomTypes,MoveAtom,N,L,Cut)
    Pos[MoveAtom]=Pos[MoveAtom]+MaxMove*(2*np.random.rand(3)-1.0)
    Pos[MoveAtom] -= L*np.around(Pos[MoveAtom]/L)
    SingleENew=project.singleenergy(Pos,AtomTypes,MoveAtom,N,L,Cut)
    if SingleENew<SingleEOld or np.exp(-(SingleENew-SingleEOld))>np.random.random():   #accept the displacement
        BigClust=project.cluster(Pos,N,L)
        if BigClust>=windowmin and BigClust<=windowmax:
            PEnergy=PEnergy-SingleEOld+SingleENew
        else:       #move the particle back
            Pos[MoveAtom]=OldPos
            BigClust=OldBigClust
    else:       #move the particle back
        Pos[MoveAtom]=OldPos

    return Pos,PEnergy,BigClust

def Insert(Pos,AtomTypes,N,PEnergy,windowmin,windowmax,BigClust):    #insert an atom pair
    OldBigClust=BigClust
    N=N+2
    Pos[N-2]=L*np.random.rand(3)-L/2
    AtomTypes[N-2]=2*np.random.randint(0,2)-1
    Pos[N-1]=L*np.random.rand(3)-L/2
    AtomTypes[N-1]= (-1)*AtomTypes[N-2]
    if AtomTypes[N-2]==AtomTypes[N-1]:
        print "Charges are same"
    Eone=project.singleenergy(Pos,AtomTypes,N-2,N,L,Cut)
    Etwo=project.singleenergy(Pos,AtomTypes,N-1,N,L,Cut)
    DeltaE = Eone + Etwo
    arg = -DeltaE + BetaMu + np.log(V/N)
    if arg > 0 or np.exp(arg)>np.random.random():     #keep the insertion; update energy
        BigClust=project.cluster(Pos,N,L)
        if BigClust>=windowmin and BigClust<=windowmax:
            PEnergy = PEnergy + DeltaE
        else:
            Pos[N-1]=np.zeros(3,float)
            Pos[N-2]=np.zeros(3,float)
            AtomTypes[N-1]=0
            AtomTypes[N-2]=0
            N=N-2
            BigClust=OldBigClust
    else:       #reverse the move
        Pos[N-1]=np.zeros(3,float)
        Pos[N-2]=np.zeros(3,float)
        AtomTypes[N-1]=0
        AtomTypes[N-2]=0
        N=N-2
    return Pos,AtomTypes,N,PEnergy,BigClust            
        
def Delete(Pos,AtomTypes,N,PEnergy,windowmin,windowmax,BigClust):    #delete an atom pair
    OldPos=Pos
    OldAtomTypes=AtomTypes
    OldN=N
    OldPEnergy=PEnergy
    OldBigClust=BigClust
    DAtomOne=np.random.randint(N)
    DAtomTwo=np.random.randint(N)
    while AtomTypes[DAtomTwo]==AtomTypes[DAtomOne]:
        DAtomTwo=np.random.randint(N)
    if AtomTypes[DAtomOne]==AtomTypes[DAtomTwo]:
        print "Charges are same"
    if DAtomTwo==DAtomOne:
        print "DAtomOne is same as DAtomTwo"
    Eone=project.singleenergy(Pos,AtomTypes,DAtomOne,N,L,Cut)
    Etwo=project.singleenergy(Pos,AtomTypes,DAtomTwo,N,L,Cut)
    DeltaE = -Eone-Etwo
    arg = -DeltaE - BetaMu + np.log(N/V)
    if arg > 0 or np.exp(arg)>np.random.random():     #delete
        if DAtomOne != (N-1) and DAtomOne!=(N-2) and DAtomTwo!=(N-1) and DAtomTwo!=(N-2):
            Pos[DAtomOne]=Pos[N-1]
            Pos[DAtomTwo]=Pos[N-2]
            Pos[N-1]=np.zeros(3,float)
            Pos[N-2]=np.zeros(3,float)
            AtomTypes[DAtomOne]=AtomTypes[N-1]
            AtomTypes[DAtomTwo]=AtomTypes[N-2]
            AtomTypes[N-1]=0
            AtomTypes[N-2]=0
        elif DAtomOne == (N-1):
            if DAtomTwo == (N-2):
                Pos[DAtomOne]=np.zeros(3,float)
                AtomTypes[DAtomOne]=0
                Pos[DAtomTwo]=np.zeros(3,float)
                AtomTypes[DAtomTwo]=0
            else:            #so DAtomOne is N-1, but DAtomTwo is not one of the last two
                Pos[DAtomTwo]=Pos[N-2]
                AtomTypes[DAtomTwo]=AtomTypes[N-2]
                Pos[N-2]=np.zeros(3,float)
                AtomTypes[N-2]=0
                Pos[N-1]=np.zeros(3,float)
                AtomTypes[N-1]=0
        elif DAtomOne == (N-2):
            if DAtomTwo== (N-1):
                Pos[DAtomOne]=np.zeros(3,float)
                AtomTypes[DAtomOne]=0
                Pos[DAtomTwo]=np.zeros(3,float)
                AtomTypes[DAtomTwo]=0
            else:            #so DAtomOne is N-2, but DAtomTwo is not one of the last two
                Pos[DAtomTwo]=Pos[N-1]
                AtomTypes[DAtomTwo]=AtomTypes[N-1]
                Pos[N-1]=np.zeros(3,float)
                AtomTypes[N-1]=0
                Pos[N-2]=np.zeros(3,float)
                AtomTypes[N-2]=0  
        elif DAtomTwo==(N-1):   #DAtomOne is not one of the last two
            Pos[DAtomOne]=Pos[N-2]
            AtomTypes[DAtomOne]=AtomTypes[N-2]
            Pos[N-2]=np.zeros(3,float)
            AtomTypes[N-2]=0
            Pos[N-1]=np.zeros(3,float)
            AtomTypes[N-1]=0            
        elif DAtomTwo==(N-2):   #DAtomOne is not one of the last two
            Pos[DAtomOne]=Pos[N-1]
            AtomTypes[DAtomOne]=AtomTypes[N-1]
            Pos[N-1]=np.zeros(3,float)
            AtomTypes[N-1]=0
            Pos[N-2]=np.zeros(3,float)
            AtomTypes[N-2]=0               
        else:
            print "Something Go Bad"
        N=N-2
        PEnergy=PEnergy +DeltaE
    BigClust=project.cluster(Pos,N,L)
    if BigClust>windowmax or BigClust<windowmin:    
        Pos=OldPos
        AtomTypes=OldAtomTypes
        N=OldN
        PEnergy=OldPEnergy
        BigClust=OldBigClust
    return Pos,AtomTypes,N,PEnergy,BigClust

#### MODULE LEVEL CODE ####
ProdSweeps=300
EquilSweeps=135 
numtrials=35
UpdateFreq=20
z=file("outputA.txt","w")
z.write('sweep \t PEnergy \t N \t NetCharge \t BigCluster \n')
Hist=file("Histogram.txt","w")
Hist.write('bin \t frequency \n')
Names=[]
BetaMu=-2.2
width=5
Histogram=np.zeros(NMax,int)
for trial in range(numtrials):  #many replicas to average over
    windowmin=(trial+1)*width+20
    windowmax=(trial+2)*width+20
    Histogram[windowmin]=0
    N=windowmin+2
    Pos,AtomTypes=InitNucleus(N,L)      
    #Pos,AtomTypes=InitRandom(N,L)
    PEnergy=project.calcenergy(Pos,AtomTypes,N,L,Cut)
    BigClust=project.cluster(Pos,N,L)
    for sweep in range(EquilSweeps+ProdSweeps):
        #if sweep%UpdateFreq==0:    #update total energy
        #    PEnergy=project.calcenergy(Pos,AtomTypes,N,L,Cut)
        if sweep>EquilSweeps:
            Charge=np.sum(AtomTypes)
            z.write(str(sweep))
            z.write('\t')
            z.write(str(PEnergy))
            z.write('\t')
            z.write(str(N))
            z.write('\t')
            z.write(str(Charge))
            z.write('\t')
            z.write(str(BigClust))
            z.write('\n')
            Histogram[BigClust]+=1
            print sweep,  N,  BigClust, PEnergy
        for move in range(200):
            randy=np.random.randint(10)
            if randy<6:
                Pos,PEnergy,BigClust = Displace(Pos,AtomTypes,PEnergy,windowmin,windowmax,BigClust)
            elif randy<8:
                Pos,AtomTypes,N,PEnergy,BigClust = Insert(Pos,AtomTypes,N,PEnergy,windowmin,windowmax,BigClust)
            else:
                Pos,AtomTypes,N,PEnergy,BigClust = Delete(Pos,AtomTypes,N,PEnergy,windowmin,windowmax,BigClust)
        
    for c in range(windowmin,(windowmax+1)):
        Hist.write(str(c))
        Hist.write('\t')
        Hist.write(str(Histogram[c]))
        Hist.write('\n')
    PosWrite=np.zeros((N,3),float)


z.close()
Hist.close()
